import React, { FC, useCallback, useEffect } from 'react';
import { Gesture, GestureDetector } from 'react-native-gesture-handler';
import Animated, { runOnJS, useAnimatedStyle, useSharedValue, withSpring } from 'react-native-reanimated';
import { Dimensions, StyleSheet, View } from 'react-native';
import { getThemeColorByDefaultColor } from '../../../../../../utils';
import { borderRadiusSizes } from '../../../../../constants';

interface Props {
  isVisible?: boolean;
  enableDropDownClose?: boolean;
  showDragBar?: boolean;
  defaultHeight: string | number;
  onClose: () => void;
  children: JSX.Element | JSX.Element[];
}

export const BottomSheet: FC<Props> = ({
  isVisible,
  enableDropDownClose = true,
  showDragBar = true,
  defaultHeight,
  onClose,
  children,
}) => {
  const windowHeight = Dimensions.get('window').height;
  const MAX_TRANSLATE_Y = -windowHeight + 50;
  const modalHeight: number = typeof defaultHeight === 'string'
    ? windowHeight * (parseFloat(defaultHeight)/100)
    : defaultHeight;
  const topPosition = ((windowHeight - (windowHeight - modalHeight))) * -1;
  const translateY = useSharedValue(0);
  const context = useSharedValue({ y: 0 });

  const scrollTo = useCallback((destination: number) => {
    translateY.value = withSpring(destination, { damping: 50 });
  }, []);

  const forceClose = () => {
    runOnJS(onClose)();
    scrollTo(0);
  };

  const gesture = Gesture.Pan()
    .onStart(() => {
      context.value = { y: translateY.value };
    })
    .onUpdate((event) => {
      translateY.value = event.translationY + context.value.y;
      translateY.value = Math.max(translateY.value, MAX_TRANSLATE_Y);
    })
    .onEnd(() => {
      const lastPosition = (translateY.value) * -1;
      // lastPosition < (modalHeight/1.5) && enableDropDownClose ? forceClose() :  scrollTo(topPosition);
      lastPosition < (modalHeight/1.5) && enableDropDownClose ? runOnJS(forceClose)() :  runOnJS(scrollTo)(topPosition);
      // setLastPosition(lastPosition);
      //const halfSize = difference
      // if (translateY.value > -SCREEN_HEIGHT / 3) {
      //   scrollTo(0);
      // } else if (translateY.value < -SCREEN_HEIGHT / 1.5) {
      //   scrollTo(MAX_TRANSLATE_Y);
      // }
    });

  const rBottomSheetStyle = useAnimatedStyle(() => {
    return {
      transform: [{ translateY: translateY.value }],
    };
  });

  useEffect(() => {
    isVisible && scrollTo(topPosition);
    !isVisible && forceClose();
  }, [isVisible]);

  useEffect(() => {
    //isVisible && scrollTo(-difference);
  }, []);

  return (
    <GestureDetector gesture={gesture} >
      <Animated.View
        style={[styles().bottomSheetContainer, rBottomSheetStyle, { height: windowHeight, top: windowHeight }]}
      >
        <>
          {showDragBar && <View style={styles().line}/>}
          {children}
        </>
      </Animated.View>
    </GestureDetector>
  );
};

const styles = () => StyleSheet.create({
  bottomSheetContainer: {
    //height: SCREEN_HEIGHT,
    width: '100%',
    backgroundColor: getThemeColorByDefaultColor('background', 'extraLight'),
    position: 'absolute',
    //top: SCREEN_HEIGHT,
    borderRadius: borderRadiusSizes.medium,
  },
  line: {
    width: 75,
    height: 4,
    backgroundColor: 'grey',
    alignSelf: 'center',
    marginVertical: 15,
    borderRadius: 2,
  },
});
